@@ -1,13 +1,10 @@
 import json
 from moxn.telemetry.client import TelemetryClient
-from moxn.telemetry.transport import APITelemetryTransport, TelemetryTransportBackend
 import logging
 from functools import lru_cache
 from pathlib import Path
-
+from typing import Callable, Coroutine
 import httpx
-from pydantic import Field, HttpUrl, SecretStr
-from pydantic_settings import BaseSettings, SettingsConfigDict
 
 from moxn.exceptions import MoxnSchemaValidationError
 from moxn.models.prompt import Prompt, PromptInstance, RenderableModel
@@ -24,35 +21,17 @@ from moxn.base_models.telemetry import (
     SignedUrlResponse,
 )
 from contextlib import asynccontextmanager
-from typing import Any, AsyncGenerator, Literal, Optional
+from typing import Any, AsyncGenerator, Literal
 from uuid import UUID
 
+from moxn.telemetry.backend import HttpTelemetryBackend
 from moxn.telemetry.context import SpanContext
+from moxn.settings import MoxnSettings
 
 logger = logging.getLogger(__name__)
 
 
-class MoxnSettings(BaseSettings):
-    """Configuration settings for the Moxn client."""
-
-    user_id: str
-    org_id: str | None = None
-    api_key: SecretStr
-    base_api_route: HttpUrl = Field(
-        default=HttpUrl("https://marksweissma--moxn-api-fastapi-app.modal.run")
-    )
-    timeout: float = Field(default=60.0, description="Prompt timeout in seconds")
-
-    model_config = SettingsConfigDict(
-        frozen=True,
-        extra="forbid",
-        env_file=[".env.local", ".env"],
-        env_prefix="MOXN_",
-        env_file_encoding="utf-8",
-    )
-
-
-class MoxnClient(TelemetryTransportBackend):
+class MoxnClient:
     """
     Moxn API client for interacting with the Moxn platform.
 
@@ -87,9 +66,8 @@ class MoxnClient(TelemetryTransportBackend):
         self._polling_manager: PollingManager | None = None
         self._context_depth = 0  # Track nested context usage
 
-        # Create telemetry components with self as the backend
-        transport = APITelemetryTransport(backend=self)
-        self.telemetry_client = TelemetryClient(transport=transport)
+        backend = HttpTelemetryBackend.from_settings(self.settings)
+        self.telemetry_client = TelemetryClient(backend=backend)
 
     @property
     def client(self) -> httpx.AsyncClient:
@@ -107,7 +85,7 @@ class MoxnClient(TelemetryTransportBackend):
         )
 
     @lru_cache(maxsize=1)
-    def get_headers(self, salt: bool = True) -> dict:
+    def get_headers(self, _: bool = True) -> dict:
         """Returns the default headers for API prompts."""
         headers = {
             "x-api-key": self.settings.api_key.get_secret_value(),
@@ -126,6 +104,9 @@ class MoxnClient(TelemetryTransportBackend):
     async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
         self._context_depth -= 1
         if self._context_depth == 0:
+            # ensure all telemetry is sent before tear‐down
+            await self.telemetry_client._dispatcher.flush()
+            await self.telemetry_client.aclose()
             await self.stop_polling()
             await self.close()
 
@@ -517,7 +498,7 @@ class MoxnClient(TelemetryTransportBackend):
         prompt_instance: PromptInstance,
         name: str | None = None,
         kind: Literal["llm", "tool", "agent"] = "llm",
-        attributes: Optional[dict[str, Any]] = None,
+        attributes: dict[str, Any] | None = None,
     ) -> AsyncGenerator[SpanContext, None]:
         """
         Creates a new span context and sets it as the current span.
@@ -537,10 +518,17 @@ class MoxnClient(TelemetryTransportBackend):
         ) as span_context:
             yield span_context
 
-    async def log_event(
+    # deprecate
+    @property
+    def log_event(
+        self,
+    ) -> Callable[[LLMEvent, UUID | None], Coroutine[None, None, None]]:
+        return self.log_telemetry_event
+
+    async def log_telemetry_event(
         self,
         event: LLMEvent,
-        span_id: Optional[UUID] = None,
+        span_id: UUID | None = None,
     ) -> None:
         """
         Logs an LLM interaction event within the current span.
@@ -551,15 +539,17 @@ class MoxnClient(TelemetryTransportBackend):
         """
         await self.telemetry_client.log_event(event=event, span_id=span_id)
 
-    async def send_telemetry_log_and_get_signed_url(
-        self, request: SignedUrlRequest
+    async def send_telemetry_log_event_and_get_signed_url(
+        self, signed_url_request: SignedUrlRequest
     ) -> SignedUrlResponse:
         """Send telemetry log and get a signed URL for uploading large data."""
-        logger.debug(f"Sending telemetry log and getting signed URL: {request}")
+        logger.debug(
+            f"Sending telemetry log and getting signed URL: {signed_url_request}"
+        )
 
         try:
             # Ensure proper serialization of all fields
-            json_data = request.model_dump(
+            json_data = signed_url_request.model_dump(
                 exclude_none=True,
                 mode="json",
                 by_alias=True,
@@ -579,3 +569,10 @@ class MoxnClient(TelemetryTransportBackend):
         except Exception as e:
             logger.error(f"Signed URL request failed: {e}", exc_info=True)
             raise RuntimeError("Failed to get signed URL for telemetry data") from e
+
+    async def flush(self, timeout: float | None = 30.0) -> None:
+        """
+        Await in‐flight telemetry logs (call this at process‐exit,
+        lambda return, or FASTAPI shutdown).
+        """
+        await self.telemetry_client._dispatcher.flush(timeout)
